@@ -1,4 +1,4 @@
-#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -20,15 +20,13 @@
 """
 
 template_make_tuple_function = """\
-#define MAKE_TUPLE%(argument_count)d( %(args)s ) _MAKE_TUPLE%(argument_count)d( EVAL_ORDERED_%(argument_count)d( %(args)s ) )
-
-NUITKA_MAY_BE_UNUSED static PyObject *_MAKE_TUPLE%(argument_count)d( EVAL_ORDERED_%(argument_count)d( %(argument_decl)s ) )
+NUITKA_MAY_BE_UNUSED static PyObject *MAKE_TUPLE%(argument_count)d( %(argument_decl)s )
 {
     PyObject *result = PyTuple_New( %(argument_count)d );
 
     if (unlikely( result == NULL ))
     {
-        throw _PythonException();
+        throw PythonException();
     }
 
 %(add_elements_code)s
