@@ -1,4 +1,4 @@
-#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -30,12 +30,12 @@ from .NodeBases import (
 )
 
 
-class ExpressionAssignmentTempKeeper( ExpressionChildrenHavingBase ):
+class ExpressionAssignmentTempKeeper(ExpressionChildrenHavingBase):
     kind = "EXPRESSION_ASSIGNMENT_TEMP_KEEPER"
 
     named_children = ( "source", )
 
-    def __init__( self, variable, source, source_ref ):
+    def __init__(self, variable, source, source_ref):
         assert source is not None
 
         ExpressionChildrenHavingBase.__init__(
@@ -51,72 +51,79 @@ class ExpressionAssignmentTempKeeper( ExpressionChildrenHavingBase ):
 
         assert self.variable_version != 0
 
-    def getDetail( self ):
-        return "%s from %s" % ( self.getVariableName(), self.getAssignSource() )
+    def getDetail(self):
+        return "%s from %s" % (
+            self.getVariableName(),
+            self.getAssignSource()
+        )
 
-    def getDetails( self ):
+    def getDetails(self):
         return {
             "name" : self.getVariableName()
         }
 
-    def getVariable( self ):
+    def getVariable(self):
         return self.variable
 
-    def getVariableName( self ):
+    def getVariableName(self):
         return self.variable.getName()
 
-    def getVariableVersion( self ):
+    def getVariableVersion(self):
         return self.variable_version
 
-    getAssignSource = ExpressionChildrenHavingBase.childGetter( "source" )
+    getAssignSource = ExpressionChildrenHavingBase.childGetter(
+        "source"
+    )
 
-    def computeExpression( self, constraint_collection ):
+    def computeExpression(self, constraint_collection):
         source = self.getAssignSource()
 
-        if source.willRaiseException( BaseException ):
-            return source, "new_raise", "Keeper assignment raises."
+        if source.willRaiseException(BaseException):
+            return source, "new_raise", "Temp keeper assignment source raises."
 
         constraint_collection.onVariableSet(
             assign_node = self
         )
 
-        # TODO: This should not be done here.
-        if self.variable.getReferenced().isWriteOnly():
-            return source, "new_expression", """\
-Removed useless temporary keeper assignment."""
-
         return self, None, None
 
-    def mayRaiseException( self, exception_type ):
-        return self.getAssignSource().mayRaiseException( exception_type )
+    def mayRaiseException(self, exception_type):
+        return self.getAssignSource().mayRaiseException(
+            exception_type = exception_type
+        )
 
-    def willRaiseException( self, exception_type ):
-        return self.getAssignSource().willRaiseException( exception_type )
+    def willRaiseException(self, exception_type):
+        return self.getAssignSource().willRaiseException(
+            exception_type = exception_type
+        )
+
+    def getTruthValue(self):
+        return self.getAssignSource().getTruthValue()
 
 
-class ExpressionTempKeeperRef( NodeBase, ExpressionMixin ):
+class ExpressionTempKeeperRef(NodeBase, ExpressionMixin):
     kind = "EXPRESSION_TEMP_KEEPER_REF"
 
-    def __init__( self, variable, source_ref ):
+    def __init__(self, variable, source_ref):
         NodeBase.__init__( self, source_ref = source_ref )
 
         self.variable = variable
 
-    def getDetails( self ):
+    def getDetails(self):
         return {
             "name" : self.getVariableName()
         }
 
-    def getDetail( self ):
+    def getDetail(self):
         return self.getVariableName()
 
-    def getVariable( self ):
+    def getVariable(self):
         return self.variable
 
-    def getVariableName( self ):
+    def getVariableName(self):
         return self.variable.getName()
 
-    def computeExpression( self, constraint_collection ):
+    def computeExpression(self, constraint_collection):
         variable_trace = constraint_collection.getVariableCurrentTrace(
             variable = self.variable
         )
@@ -133,15 +140,15 @@ Replaced keeper variable usage for no side effects value."""
 
         return self, None, None
 
-    def mayRaiseException( self, exception_type ):
+    def mayRaiseException(self, exception_type):
         # Can't happen
         return False
 
-    def mayHaveSideEffects( self ):
+    def mayHaveSideEffects(self):
         return False
 
-    def isOrderRelevant( self ):
+    def isOrderRelevant(self):
         return True
 
-    def mayProvideReference( self ):
+    def mayProvideReference(self):
         return self.variable.getReferenced().getNeedsFree()
