@@ -1,4 +1,4 @@
-#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -17,10 +17,10 @@
 #
 """ The type1 node.
 
-This one just determines types. It's great for optimization. We may be able to predict its
-value, but knowing it. In that case, we have a builtin name reference for that type to
-convert to, or when checking the result of it, we will then know it's limited after the
-fact.
+This one just determines types. It's great for optimization. We may be able to
+predict its value, but knowing it. In that case, we have a builtin name
+reference for that type to convert to, or when checking the result of it, we
+will then know it's limited after the fact.
 
 """
 
@@ -32,10 +32,10 @@ from .NodeBases import (
 from nuitka.Builtins import builtin_names
 
 
-class ExpressionBuiltinType1( ExpressionBuiltinSingleArgBase ):
+class ExpressionBuiltinType1(ExpressionBuiltinSingleArgBase):
     kind = "EXPRESSION_BUILTIN_TYPE1"
 
-    def computeExpression( self, constraint_collection ):
+    def computeExpression(self, constraint_collection):
         value = self.getValue()
 
         if value.isCompileTimeConstant():
@@ -62,18 +62,32 @@ class ExpressionBuiltinType1( ExpressionBuiltinSingleArgBase ):
             return (
                 new_node,
                 "new_builtin",
-                "Replaced predictable type lookup with builtin type '%s'." % type_name
+                "Replaced predictable type lookup with builtin type '%s'." % (
+                    type_name
+                )
             )
 
         return self, None, None
 
+    def computeExpressionDrop(self, statement, constraint_collection):
+        from .NodeMakingHelpers import \
+          makeStatementExpressionOnlyReplacementNode
 
-class ExpressionBuiltinSuper( ExpressionChildrenHavingBase ):
+        result = makeStatementExpressionOnlyReplacementNode(
+            expression = self.getValue(),
+            node       = statement
+        )
+
+        return result, "new_statements", """\
+Removed type taking for unused result."""
+
+
+class ExpressionBuiltinSuper(ExpressionChildrenHavingBase):
     kind = "EXPRESSION_BUILTIN_SUPER"
 
     named_children = ( "type", "object" )
 
-    def __init__( self, super_type, super_object, source_ref ):
+    def __init__(self, super_type, super_object, source_ref):
         ExpressionChildrenHavingBase.__init__(
             self,
             values     = {
@@ -86,17 +100,17 @@ class ExpressionBuiltinSuper( ExpressionChildrenHavingBase ):
     getType = ExpressionChildrenHavingBase.childGetter( "type" )
     getObject = ExpressionChildrenHavingBase.childGetter( "object" )
 
-    def computeExpression( self, constraint_collection ):
+    def computeExpression(self, constraint_collection):
         # TODO: Quite some cases should be possible to predict.
         return self, None, None
 
 
-class ExpressionBuiltinIsinstance( ExpressionChildrenHavingBase ):
+class ExpressionBuiltinIsinstance(ExpressionChildrenHavingBase):
     kind = "EXPRESSION_BUILTIN_ISINSTANCE"
 
     named_children = ( "instance", "cls" )
 
-    def __init__( self, instance, cls, source_ref ):
+    def __init__(self, instance, cls, source_ref):
         ExpressionChildrenHavingBase.__init__(
             self,
             values     = {
@@ -109,11 +123,11 @@ class ExpressionBuiltinIsinstance( ExpressionChildrenHavingBase ):
     getInstance = ExpressionChildrenHavingBase.childGetter( "instance" )
     getCls = ExpressionChildrenHavingBase.childGetter( "cls" )
 
-    def computeExpression( self, constraint_collection ):
+    def computeExpression(self, constraint_collection):
         # TODO: Quite some cases should be possible to predict.
         return self, None, None
 
-    def mayProvideReference( self ):
+    def mayProvideReference(self):
         # Dedicated code returns "True" or "False" only, which requires no reference,
         # except for rich comparisons, which do.
         return False
