@@ -1,4 +1,4 @@
-#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -15,9 +15,15 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
-print( "Main importing from package" )
-
+print("Main importing nearby package")
+import pyexpat
+try:
+    print(pyexpat.defined_in_pyexpat )
+except AttributeError:
+    print("Must be Python3, where absolute imports are default.")
+print( "Main importing from package doing star import" )
 from some_package import star_importing
+print( "Main importing from package doing normal import" )
 from some_package import normal_importing
 
 print( "Done." )
